@@ -490,6 +490,41 @@ ipset_print_port(char *buf, unsigned int len,
 	return offset;
 }
 
+/**
+ * ipset_print_mark - print mark to string
+ * @buf: printing buffer
+ * @len: length of available buffer space
+ * @data: data blob
+ * @opt: the option kind
+ * @env: environment flags
+ *
+ * Print mark to output buffer.
+ *
+ * Return lenght of printed string or error size.
+ */
+int
+ipset_print_mark(char *buf, unsigned int len,
+		   const struct ipset_data *data,
+		   enum ipset_opt opt ASSERT_UNUSED,
+		   uint8_t env UNUSED)
+{
+	const uint32_t *mark;
+	int size, offset = 0;
+
+	assert(buf);
+	assert(len > 0);
+	assert(data);
+	assert(opt == IPSET_OPT_MARK || opt == IPSET_OPT_MARKMASK);
+
+	mark = ipset_data_get(data, opt);
+	assert(mark);
+
+	size = snprintf(buf, len, "0x%08llx", *mark);
+	SNPRINTF_FAILURE(size, len, offset);
+
+	return offset;
+}
+
 /**
  * ipset_print_iface - print interface element string
  * @buf: printing buffer
@@ -529,6 +564,37 @@ ipset_print_iface(char *buf, unsigned int len,
 	return offset;
 }
 
+/**
+ * ipset_print_comment - print arbitrary parameter string
+ * @buf: printing buffer
+ * @len: length of available buffer space
+ * @data: data blob
+ * @opt: the option kind
+ * @env: environment flags
+ *
+ * Print arbitrary string to output buffer.
+ *
+ * Return length of printed string or error size.
+ */
+int ipset_print_comment(char *buf, unsigned int len,
+		       const struct ipset_data *data, enum ipset_opt opt,
+		       uint8_t env UNUSED)
+{
+	const char *comment;
+	int size, offset = 0;
+
+	assert(buf);
+	assert(len > 0);
+	assert(data);
+	assert(opt == IPSET_OPT_ADT_COMMENT);
+
+	comment = ipset_data_get(data, opt);
+	assert(comment);
+	size = snprintf(buf + offset, len, "\"%s\"", comment);
+	SNPRINTF_FAILURE(size, len, offset);
+	return offset;
+}
+
 /**
  * ipset_print_proto - print protocol name
  * @buf: printing buffer
@@ -678,18 +744,20 @@ ipset_print_proto_port(char *buf, unsigned int len,
 		case IPPROTO_UDPLITE:
 			break;
 		case IPPROTO_ICMP:
-			return ipset_print_icmp(buf + offset, len, data,
+			size = ipset_print_icmp(buf + offset, len, data,
 						IPSET_OPT_PORT, env);
+			goto out;
 		case IPPROTO_ICMPV6:
-			return ipset_print_icmpv6(buf + offset, len, data,
+			size = ipset_print_icmpv6(buf + offset, len, data,
 						  IPSET_OPT_PORT, env);
+			goto out;
 		default:
 			break;
 		}
 	}
 	size = ipset_print_port(buf + offset, len, data, IPSET_OPT_PORT, env);
+out:
 	SNPRINTF_FAILURE(size, len, offset);
-
 	return offset;
 }
 
@@ -825,6 +893,7 @@ ipset_print_data(char *buf, unsigned int len,
 	case IPSET_OPT_GC:
 	case IPSET_OPT_HASHSIZE:
 	case IPSET_OPT_MAXELEM:
+	case IPSET_OPT_MARKMASK:
 	case IPSET_OPT_NETMASK:
 	case IPSET_OPT_PROBES:
 	case IPSET_OPT_RESIZE:
